#ifndef EA_VEC
#define EA_VEC

/* vectors.h AuerSoft 15.6.1997, mostly type-neutral macros in C. */
/* Normally using 3d vectors. Some macros also have 2d versions.  */

typedef double V_FP; /* the TYPE used for all coordinates */

typedef struct {
  V_FP x,y,z;
} VEC; /* 3d vector */

/* take the square of a number */
#define sqr(x) ((x) * (x)) /* or pow(x,2) */

/* the product of two vectors is a number */
#define vMULv(u,v) ( \
  ((u).x * (v).x) + \
  ((u).y * (v).y) + \
  ((u).z * (v).z) )

/* the special square root of the input, special sign treatment */
#define xABS(x) ( (x>0) ? sqrt(x) : -sqrt(-(x)) ) /* keep sgn */
/* x is used 2 times, so it better be no macro. */

/* *** make this a function, not a macro, if optimization fails: */
/* determine the length of the vector */
#define vABS(v) xABS( vMULv((v), (v)) )

/* scale a vector by a factor */
#define vMULs(v,s) { (v).x *= (s); (v).y *= (s); (v).z *= (s); }

/* scale a TWO dimensional vector by a factor */
#define v2MULs(v,s) { (v).x *= (s); (v).y *= (s); }

/* *** obviously, w may not be identical to u or v here: */
/* the cross product is orthogonal to the input vectors. */
/* make w the cross product of u and v */
#define vCROSSv(u,v,w) { \
 (w).x = ((u).y * (v).z) - ((u).z * (v).y); \
 (w).y = ((u).z * (v).x) - ((u).x * (v).z); \
 (w).z = ((u).x * (v).y) - ((u).y * (v).x); }

/* subtract v from u */
#define vSUBv(u,v) { (u).x -= (v).x; (u).y -= (v).y; (u).z -= (v).z; }

/* subtract v from u */
#define v2SUBv(u,v) { (u).x -= (v).x; (u).y -= (v).y; }

/* add v to u */
#define vADDv(u,v) { (u).x += (v).x; (u).y += (v).y; (u).z += (v).z; }

/* add v to u */
#define v2ADDv(u,v) { (u).x += (v).x; (u).y += (v).y; }

/* *** obviously, v may not be identical to u here: */
/* set v to u multiplied by matrix v1 v2 v3 */
#define vMATvvv(u,v,v1,v2,v3) { \
 (v).x = vMULv((u), (v1)); \
 (v).y = vMULv((u), (v2)); \
 (v).z = vMULv((u), (v3)); }

/* copy the vector u to vector v */
#define vCOPYv(v,u) { (v).x = (u).x; (v).y = (u).y; (v).z = (u).z; }

/* copy the TWO dimensional vector u to vector v */
#define v2COPYv(v,u) { (v).x = (u).x; (v).y = (u).y; }

/* *** obviously, x1 may not be identical to x2 here: */
/* normally, sinphi and cosphi are sin and cos of the same angle */
/* rotate x2 y2 using sinphi and cosphi, to create x1 y1 */
#define ROT2dMAT(x1,y1,x2,y2,sinphi,cosphi) { \
 (x1) = ( (x2) * cosphi) + ((y2) * sinphi); \
 (y1) = (-(x2) * sinphi) + ((y2) * cosphi); }

#endif
